@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -34,32 +30,28 @@
 # /usr/share/common-licenses/AGPL-3; if not, seGe
 # <https://www.gnu.org/licenses/>.
 
-from os import listdir
 import os.path
 import traceback
-
-# strptime import is not threadsafe
-# Workaround: import early
-# Upstream bug: https://bugs.python.org/issue7980
-# UCS bug: https://forge.univention.org/bugzilla/show_bug.cgi?id=45585
-import _strptime
-
-from univention.management.console.modules import Base
-from univention.management.console.modules.decorators import simple_response, sanitize
-from univention.management.console.modules.sanitizers import PatternSanitizer, DictSanitizer, StringSanitizer
-from univention.management.console.modules.mixins import ProgressMixin
-from univention.management.console.log import MODULE
-from univention.management.console.modules.diagnostic import plugins
 from collections import OrderedDict
+from os import listdir
+from typing import Any, Callable, Dict, Iterator, List, Optional, Pattern, Match
+
 from univention.lib.i18n import Translation
+from univention.management.console.log import MODULE
+from univention.management.console.modules import Base
+from univention.management.console.modules.decorators import sanitize, simple_response
+from univention.management.console.modules.diagnostic import plugins
+from univention.management.console.modules.mixins import ProgressMixin
+from univention.management.console.modules.sanitizers import (
+	DictSanitizer, PatternSanitizer, StringSanitizer,
+)
 
-_strptime
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 
 class Problem(Exception):
 
-	def __init__(self, description=None, **kwargs):
+	def __init__(self, description: str = "", **kwargs: Any) -> None:
 		super(Problem, self).__init__(description)
 		self.kwargs = kwargs
 		kwargs['type'] = self.__class__.__name__.lower()
@@ -92,8 +84,8 @@ class Instance(Base, ProgressMixin):
 
 	PLUGIN_DIR = os.path.dirname(plugins.__file__)
 
-	def init(self):
-		self.modules = {}
+	def init(self) -> None:
+		self.modules: Dict[str, "Plugin"] = {}
 		self.load()
 
 	@sanitize(
@@ -101,32 +93,32 @@ class Instance(Base, ProgressMixin):
 		args=DictSanitizer({})
 	)
 	@simple_response(with_progress=True)
-	def run(self, plugin, args=None):
-		plugin = self.get(plugin)
-		MODULE.process('Running %s' % (plugin,))
-		for line in plugin.run_descr:
+	def run(self, plugin: str, args: Any = None):
+		plug = self.get(plugin)
+		MODULE.process('Running %s' % (plug,))
+		for line in plug.run_descr:
 			MODULE.process(line)
 		args = args or {}
 
-		return plugin.execute(self, **args)
+		return plug.execute(self, **args)
 
-	def new_progress(self, *args, **kwargs):
+	def new_progress(self, *args: Any, **kwargs: Any) -> Any:
 		progress = super(Instance, self).new_progress(*args, **kwargs)
 		progress.retry_after = 600
 		return progress
 
 	@sanitize(pattern=PatternSanitizer(default='.*'))
 	@simple_response
-	def query(self, pattern):
+	def query(self, pattern: Pattern[str]) -> List[Dict[str, Any]]:
 		return [plugin.dict for plugin in self if plugin.match(pattern)]
 
 	@property
-	def plugins(self):
+	def plugins(self) -> Iterator[str]:
 		for plugin in listdir(self.PLUGIN_DIR):
 			if plugin.endswith('.py') and plugin != '__init__.py':
 				yield plugin[:-3]
 
-	def load(self):
+	def load(self) -> None:
 		for plugin in self.plugins:
 			try:
 				self.modules[plugin] = Plugin(plugin)
@@ -135,10 +127,10 @@ class Instance(Base, ProgressMixin):
 				raise
 		self.modules = OrderedDict(sorted(self.modules.items(), key=lambda t: t[0]))
 
-	def get(self, plugin):
+	def get(self, plugin: str) -> "Plugin":
 		return self.modules[plugin]
 
-	def __iter__(self):
+	def __iter__(self) -> Iterator["Plugin"]:
 		return iter(self.modules.values())
 
 
@@ -222,67 +214,62 @@ class Plugin(object):
 		+-- ProblemFixed
 	"""
 	@property
-	def title(self):
+	def title(self) -> str:
 		u"""A title for the problem"""
 		return getattr(self.module, 'title', '')
 
 	@property
-	def description(self):
+	def description(self) -> str:
 		u"""A description of the problem and how to solve it"""
 		return getattr(self.module, 'description', '')
 
 	@property
-	def buttons(self):
+	def buttons(self) -> List[Dict[str, str]]:
 		u"""Buttons which are displayed e.g. to automatically solve the problem"""
 		return list(getattr(self.module, 'buttons', []))
 
 	@property
-	def run_descr(self):
+	def run_descr(self) -> List[str]:
 		return list(getattr(self.module, 'run_descr', []))
 
 	@property
-	def popups(self):
-		u"""Buttons with pop ups"""
-		return getattr(self.module, 'popups', [])
-
-	@property
-	def umc_modules(self):
+	def umc_modules(self) -> List[Dict[str, Any]]:
 		u"""References to UMC modules which can help solving the problem.
 			(module, flavor, properties)
 		"""
 		return getattr(self.module, 'umc_modules', [])
 
 	@property
-	def links(self):
+	def links(self) -> List[Dict[str, str]]:
 		u"""Links to e.g. related SDB articles
 			(url, link_name)
 		"""
 		return getattr(self.module, 'links', [])
 
 	@property
-	def actions(self):
+	def actions(self) -> Dict[str, Callable[[Instance], Optional[Dict[str, Any]]]]:
 		return getattr(self.module, 'actions', {})
 
-	def __init__(self, plugin):
+	def __init__(self, plugin: str) -> None:
 		self.plugin = plugin
 		self.load()
 
-	def load(self):
+	def load(self) -> None:
 		self.module = __import__(
 			'univention.management.console.modules.diagnostic.plugins.%s' % (self.plugin,),
 			fromlist=['univention.management.console.modules.diagnostic'],
 			level=0
 		)
 
-	def execute(self, umc_module, action=None, **kwargs):
+	def execute(self, umc_module: Instance, action=None, **kwargs: Any) -> Dict[str, Any]:
 		success = True
-		errors = {}
+		errors: Dict[str, Any] = {}
 		execute = self.actions.get(action, self.module.run)
 		try:
 			try:
-				result = execute(umc_module, **kwargs)
-				if isinstance(result, dict):
-					errors.update(result)
+				ret = execute(umc_module, **kwargs)
+				if isinstance(ret, dict):
+					errors.update(ret)
 			except Problem:
 				raise
 			except Exception:
@@ -291,34 +278,33 @@ class Plugin(object):
 			success = False
 			errors.update(exc.kwargs)
 
-		result = dict(
-			success=success,
-			type='success'
-		)
+		result: Dict[str, Any] = {
+			"success": success,
+			"type": 'success',
+		}
 		result.update(self.dict)
 		result.update(errors)
 		result.setdefault('buttons', []).insert(0, {'label': _('Test again')})
 		return result
 
-	def match(self, pattern):
+	def match(self, pattern: Pattern[str]) -> Optional[Match]:
 		return pattern.match(self.title) or pattern.match(self.description)
 
-	def __str__(self):
+	def __str__(self) -> str:
 		return '%s' % (self.plugin,)
 
 	@property
-	def dict(self):
-		return dict(
-			id=str(self),
-			plugin=str(self),
-			title=self.title,
-			description=self.description,
-			umc_modules=self.umc_modules,
-			links=self.links,
-			buttons=self.buttons,
-			popups=self.popups,
-		)
+	def dict(self) -> Dict[str, Any]:
+		return {
+			"id": str(self),
+			"plugin": str(self),
+			"title": self.title,
+			"description": self.description,
+			"umc_modules": self.umc_modules,
+			"links": self.links,
+			"buttons": self.buttons,
+		}
 
 
-def main():
+def main() -> None:
 	print('TODO: someday implement?')
