@@ -1,25 +1,53 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
+#
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
+# Copyright 2016-2022 Univention GmbH
+#
+# https://www.univention.de/
+#
+# All rights reserved.
+#
+# The source code of this program is made available
+# under the terms of the GNU Affero General Public License version 3
+# (GNU AGPL V3) as published by the Free Software Foundation.
+#
+# Binary versions of this program provided by Univention to you as
+# well as other copyrighted, protected or trademarked materials like
+# Logos, graphics, fonts, specific documentations and configurations,
+# cryptographic keys etc. are subject to a license agreement between
+# you and Univention and not subject to the GNU AGPL V3.
+#
+# In the case you use this program under the terms of the GNU AGPL V3,
+# the program is provided in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License with the Debian GNU/Linux or Univention distribution in file
+# /usr/share/common-licenses/AGPL-3; if not, see
+# <https://www.gnu.org/licenses/>.
 
-from univention.management.console.config import ucr
-from univention.management.console.modules.diagnostic import Critical
-from univention.management.console.modules.diagnostic import Warning
-from univention.admin import uldap
-from univention.admin import modules
-from univention.management.console.log import MODULE
-import paramiko
 import logging
-import socket
 import re
+import socket
 
+import paramiko
+
+from univention.admin import modules, uldap
 from univention.lib.i18n import Translation
+from univention.management.console.config import ucr
+from univention.management.console.log import MODULE
+from univention.management.console.modules.diagnostic import Critical, Instance, Warning
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('SSH connection to UCS server failed!')
 
-ucr.load()
-fqdn = ".".join((ucr['hostname'], ucr['domainname']))
-run_descr = ['This can be checked by running:  univention-ssh /etc/machine.secret "%s$@%s" echo OK' % (ucr["hostname"], fqdn)]
+FQDN = "%(hostname)s.%(domainname)s" % ucr
+run_descr = ['This can be checked by running:  univention-ssh /etc/machine.secret "%s$@%s" echo OK' % (ucr["hostname"], FQDN)]
 
 
 class IgnorePolicy(paramiko.MissingHostKeyPolicy):
@@ -28,11 +56,11 @@ class IgnorePolicy(paramiko.MissingHostKeyPolicy):
 		pass
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	# Now a workaround for paramico logging to connector-s4.log
 	# because one of the diagnostic plugins instantiates s4connector.s4.s4()
 	# which initializes univention.debug2, which initializes logging.basicConfig
-	logger = paramiko.util.logging.getLogger()
+	logger = logging.getLogger("paramiko")
 	logger.setLevel(logging.CRITICAL)
 
 	try:
@@ -58,7 +86,7 @@ def run(_umc_instance):
 			host.open()
 			ucs_hosts.append(host['name'])
 
-	with open('/etc/machine.secret', 'rb') as fd:
+	with open('/etc/machine.secret', 'r') as fd:
 		password = fd.read().strip()
 
 	gen_msg = _('The ssh connection to at least one other UCS server failed. ')
@@ -72,13 +100,14 @@ def run(_umc_instance):
 	auth_info = _('Login to the remote server with the uid %(uid)s and the password from /etc/machine.secret failed. ')
 	auth_info += _('Please check /var/log/auth.log on the remote server for further information.')
 
-	bad = dict()
+	bad = {}
 	key_failed = False
 	auth_failed = False
-	data = dict(
-		fqdn=ucr['hostname'] + '.' + ucr['domainname'],
-		uid=ucr['hostname'] + '$',
-		hostname=ucr['hostname'])
+	data = {
+		"fqdn": FQDN,
+		"uid": ucr['hostname'] + '$',
+		"hostname": ucr['hostname']
+	}
 
 	for host in ucs_hosts:
 		client = paramiko.SSHClient()
