@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -34,10 +30,11 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-from subprocess import Popen, PIPE, STDOUT
-from univention.management.console.modules.diagnostic import Critical, MODULE
+from subprocess import PIPE, STDOUT, Popen
 
 from univention.lib.i18n import Translation
+from univention.management.console.modules.diagnostic import MODULE, Critical, Instance
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check join status')
@@ -51,7 +48,7 @@ umc_modules = [{'module': 'join'}]
 run_descr = ['This can be checked by running: univention-check-join-status']
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	process = Popen(['univention-check-join-status'], stdout=PIPE, stderr=STDOUT)
 	(stdout, stderr) = process.communicate()
 	if process.returncode != 0:
