@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -34,18 +30,19 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
+import glob
+import grp
 import os
 import pwd
-import grp
 import stat
-import glob
 from collections import namedtuple
-from univention.management.console.log import MODULE
-
-import univention.config_registry
-from univention.management.console.modules.diagnostic import Warning
+from typing import Iterator, Tuple
 
+from univention.config_registry import ucr_live as configRegistry
 from univention.lib.i18n import Translation
+from univention.management.console.log import MODULE
+from univention.management.console.modules.diagnostic import Instance, Warning
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check file permissions')
@@ -54,38 +51,38 @@ run_descr = ['Checks file permissions']
 
 
 class CheckError(Exception):
-	def __init__(self, filename):
+	def __init__(self, filename: str) -> None:
 		self.filename = filename
 
 
 class DoesNotExist(CheckError):
-	def __str__(self):
+	def __str__(self) -> str:
 		return _('File {path!r} does not exist.'.format(path=self.filename))
 
 
 class OwnerMismatch(CheckError):
-	def __init__(self, filename, expected_owner, actual_owner):
+	def __init__(self, filename: str, expected_owner: Tuple[str, str], actual_owner: Tuple[str, str]) -> None:
 		super(OwnerMismatch, self).__init__(filename)
 		self.expected_owner = expected_owner
 		self.actual_owner = actual_owner
 
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('File {path!r} has owner {actual!r} while {expected!r} was expected.')
 		return msg.format(path=self.filename, expected=':'.join(self.expected_owner), actual=':'.join(self.actual_owner))
 
 
 class PermissionMismatch(CheckError):
-	def __init__(self, filename, actual_mode, expected_mode):
+	def __init__(self, filename: str, actual_mode: int, expected_mode: int) -> None:
 		super(PermissionMismatch, self).__init__(filename)
 		self.actual_mode = actual_mode
 		self.expected_mode = expected_mode
 
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('File {path!r} has mode {actual:o}, {expected:o} was expected.')
 		return msg.format(path=self.filename, actual=self.actual_mode, expected=self.expected_mode)
 
 
-def get_actual_owner(uid, gid):
+def get_actual_owner(uid: int, gid: int) -> Tuple[str, str]:
 	try:
 		name = pwd.getpwuid(uid).pw_name
 	except KeyError:
@@ -97,38 +94,33 @@ def get_actual_owner(uid, gid):
 	return (name, group)
 
 
-def check_file(path, owner, group, mode, must_exist=False):
+def check_file(path: str, owner: str, group: str, mode: int, must_exist: bool = False) -> Iterator[CheckError]:
 	try:
 		file_stat = os.stat(path)
-	except EnvironmentError:
+	except OSError:
 		if must_exist:
 			MODULE.error("%s must exist, but does not" % (path))
-			return DoesNotExist(path)
-		return True
+			yield DoesNotExist(path)
+		return
 
 	expected_owner = (owner, group)
 	actual_owner = get_actual_owner(file_stat.st_uid, file_stat.st_gid)
 	if expected_owner != actual_owner:
 		MODULE.error("Owner mismatch: %s should be owned by %s, is actually owned by %s" % (path, expected_owner, actual_owner))
-		return OwnerMismatch(path, expected_owner, actual_owner)
+		yield OwnerMismatch(path, expected_owner, actual_owner)
 
 	actual_mode = stat.S_IMODE(file_stat.st_mode)
 	if actual_mode != mode:
 		MODULE.error("Permission mismatch: %s should have the permission mode %s but has the mode %s" % (path, mode, actual_mode))
-		return PermissionMismatch(path, actual_mode, mode)
-
-	return True
+		yield PermissionMismatch(path, actual_mode, mode)
 
 
-def file_and_permission_checks():
-	configRegistry = univention.config_registry.ConfigRegistry()
-	configRegistry.load()
-
+def file_and_permission_checks() -> Iterator[CheckError]:
 	is_primary = configRegistry.get('server/role') in ('domaincontroller_master', 'domaincontroller_backup')
 	is_dc = configRegistry.get('server/role').startswith('domaincontroller_')
 	(host, domain) = (configRegistry.get('hostname'), configRegistry.get('domainname'))
 
-	cf_type = namedtuple('check_file_kwargs', ('path', 'owner', 'group', 'mode', 'must_exist'))
+	cf_type = namedtuple('cf_type', ('path', 'owner', 'group', 'mode', 'must_exist'))
 
 	check_file_args = [
 		cf_type('/etc/ldap.secret', 'root', 'DC Backup Hosts', 0o640, must_exist=is_primary),
@@ -182,11 +174,12 @@ def file_and_permission_checks():
 		check_file_args.append(cf_type(saml_key, 'root', 'samlcgi', 0o640, must_exist=True))
 
 	for kwarg in check_file_args:
-		yield check_file(*kwarg)
+		for error in check_file(*kwarg):
+			yield error
 
 
-def run(_umc_instance):
-	error_descriptions = [str(error) for error in file_and_permission_checks() if isinstance(error, CheckError)]
+def run(_umc_instance: Instance) -> None:
+	error_descriptions = [str(error) for error in file_and_permission_checks()]
 	if error_descriptions:
 		raise Warning(description='\n'.join(error_descriptions))
 
