@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -34,15 +30,17 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
+from typing import Dict, Iterator, List, Tuple
+
 import ldap
 
-import univention.uldap
-import univention.lib.s4 as s4
 import univention.lib.misc
-import univention.config_registry
-from univention.management.console.modules.diagnostic import Warning, MODULE
-from univention.management.console.modules.diagnostic import util
+import univention.lib.s4 as s4
+import univention.uldap
+from univention.config_registry import ucr_live as ucr
 from univention.lib.i18n import Translation
+from univention.management.console.modules.diagnostic import MODULE, Instance, Warning, util
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check well known SIDs')
@@ -50,61 +48,62 @@ description = _('All SIDs exist and names are consistent.')
 run_descr = ['Checks if all well known SIDs exist and if their names are consistent']
 
 
-NON_EXISTENT_SIDS = set((
+NON_EXISTENT_SIDS = {
 	'Power Users', 'Creator Group Server',
 	'Creator Owner Server', 'Local', 'Console Logon', 'All Services',
 	'Creator Authority', 'Local Authority', 'NT Authority',
-	'Non-unique Authority', 'Cloneable Domain Controllers'))
+	'Non-unique Authority', 'Cloneable Domain Controllers',
+}
 
 
 class CheckError(Exception):
-	def __init__(self, sid, expected_name):
+	def __init__(self, sid: str, expected_name: str) -> None:
 		self.sid = sid
 		self.expected_name = expected_name
 
 
 class SIDNotFound(CheckError):
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('No user or group with SID {sid} found, expected {expected!r}.')
 		return msg.format(sid=self.sid, expected=self.expected_name)
 
 
 class SIDMismatch(CheckError):
-	def __init__(self, sid, actual_sid, expected_name):
+	def __init__(self, sid: str, actual_sid: str, expected_name: str) -> None:
 		super(SIDMismatch, self).__init__(sid, expected_name)
 		self.actual_sid = actual_sid
 
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('User or group with name {name!r} has sid {actual_sid}, but should be {sid}.')
 		return msg.format(name=self.expected_name, actual_sid=self.actual_sid, sid=self.sid)
 
 
 class NameMismatch(CheckError):
-	def __init__(self, sid, expected_name, actual_name):
+	def __init__(self, sid: str, expected_name: str, actual_name: str) -> None:
 		super(NameMismatch, self).__init__(sid, expected_name)
 		self.actual_name = actual_name
 
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('User or group with SID {sid} has name {actual!r}, but should be {expected!r}.')
 		return msg.format(sid=self.sid, actual=self.actual_name, expected=self.expected_name)
 
 
 class LDAPConnection(object):
-	def __init__(self):
+	def __init__(self) -> None:
 		self._connection = univention.uldap.getMachineConnection()
 
-	def search(self, expression, attr=[]):
-		for (dn, attr) in self._connection.search(expression, attr=attr):
+	def search(self, expression: str, attr: List[str] = []) -> Iterator[Tuple[str, Dict[str, List[bytes]]]]:
+		for (dn, attrs) in self._connection.search(expression, attr=attr):
 			if dn is not None:
-				yield (dn, attr)
+				yield (dn, attrs)
 
-	def get_domain_sid(self):
+	def get_domain_sid(self) -> str:
 		for (dn, attr) in self.search('(objectClass=sambaDomain)', attr=['sambaSID']):
-			for sid in attr.get('sambaSID'):
+			for sid in attr['sambaSID']:
 				return sid.decode('ASCII')
 		raise KeyError('domain sid not found')
 
-	def get_by_sid(self, sid):
+	def get_by_sid(self, sid: str) -> str:
 		expression = ldap.filter.filter_format('(sambaSID=%s)', (sid,))
 		for (dn, attr) in self.search(expression, attr=['cn', 'uid']):
 			for uid in attr.get('uid', []):
@@ -113,7 +112,7 @@ class LDAPConnection(object):
 				return cn.decode('UTF-8')
 		raise KeyError(sid)
 
-	def get_by_name(self, name):
+	def get_by_name(self, name: str) -> str:
 		expression = ldap.filter.filter_format('(|(cn=%s)(uid=%s))', (name, name))
 		for (dn, attr) in self.search(expression, attr=['sambaSID']):
 			for sid in attr.get('sambaSID', []):
@@ -121,7 +120,7 @@ class LDAPConnection(object):
 		raise KeyError(name)
 
 
-def all_sids_and_names(domain_sid):
+def all_sids_and_names(domain_sid: str) -> Iterator[Tuple[str, str]]:
 	for (sid, name) in s4.well_known_sids.items():
 		if name not in NON_EXISTENT_SIDS:
 			yield (sid, name)
@@ -131,7 +130,7 @@ def all_sids_and_names(domain_sid):
 			yield ('{}-{}'.format(domain_sid, rid), name)
 
 
-def custom_name(name, ucr=None):
+def custom_name(name: str) -> str:
 	mapped_user = univention.lib.misc.custom_username(name, ucr)
 	if mapped_user != name:
 		return mapped_user
@@ -141,13 +140,11 @@ def custom_name(name, ucr=None):
 	return name
 
 
-def check_existence_and_consistency():
+def check_existence_and_consistency() -> Iterator[CheckError]:
 	ldap_connection = LDAPConnection()
 	domain_sid = ldap_connection.get_domain_sid()
-	ucr = univention.config_registry.ConfigRegistry()
-	ucr.load()
 	for (sid, expected_name) in all_sids_and_names(domain_sid):
-		mapped_name = custom_name(expected_name, ucr)
+		mapped_name = custom_name(expected_name)
 		try:
 			# The user/group retrieved by SID should have the name as specified
 			# in the well-known-sid-mapping (or mapped as per
@@ -171,7 +168,7 @@ def check_existence_and_consistency():
 				yield NameMismatch(sid, mapped_name, actual_name)
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	if not util.is_service_active('S4 Connector'):
 		return
 
