@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -36,13 +32,10 @@
 
 import psutil
 
-import univention.uldap
-
-import univention.config_registry
-from univention.management.console.modules.diagnostic import Critical, MODULE
-from univention.management.console.modules.diagnostic import util
-
+from univention.config_registry import ucr_live as configRegistry
 from univention.lib.i18n import Translation
+from univention.management.console.modules.diagnostic import MODULE, Critical, Instance, util
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check Heimdal KDC on Samba 4 DC')
@@ -51,7 +44,7 @@ umc_modules = [{'module': 'services'}]
 run_descr = ['This can be checked by running: samba-tool processes']
 
 
-def samba_kdc_running():
+def samba_kdc_running() -> bool:
 	try:
 		import samba.messaging
 	except ImportError:
@@ -64,19 +57,17 @@ def samba_kdc_running():
 	return bool(ids)
 
 
-def is_heimdal_kdc_running():
+def is_heimdal_kdc_running() -> bool:
 	kdc_paths = ('/usr/lib/heimdal-servers/kdc', '/usr/lib/heimdal-servers/kpasswdd')
 	process_paths = (p.exe() for p in psutil.process_iter())
 	return any(path in kdc_paths for path in process_paths)
 
 
-def is_kerberos_autostart_disabled():
-	configRegistry = univention.config_registry.ConfigRegistry()
-	configRegistry.load()
+def is_kerberos_autostart_disabled() -> bool:
 	return configRegistry.is_false('kerberos/autostart')
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	error = _('This is a Samba 4 DC, but `samba-tool processes` reports no `kdc_server`.')
 	heimdal_error = _('This may be, because Heimdal KDC seems to be running.')
 	autostart_error = _('This may be, because `kerberos/autostart` is not disabled.')
