@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -36,9 +32,10 @@
 
 
 import subprocess
-from univention.management.console.modules.diagnostic import Warning, MODULE
 
 from univention.lib.i18n import Translation
+from univention.management.console.modules.diagnostic import MODULE, Instance, Warning
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check for modified UCR templates')
@@ -46,7 +43,7 @@ description = _('No problems found with modified UCR templates')
 run_descr = ['This can be checked by running: univention-check-templates']
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	cmd = ['univention-check-templates']
 	try:
 		subprocess.check_output(cmd)
