@@ -1,8 +1,4 @@
 #!/usr/bin/python3
-# coding: utf-8
-#
-# Univention Management Console module:
-#  System Diagnosis UMC module
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -34,72 +30,75 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-import subprocess
 import itertools as it
+import subprocess
+from typing import Any, Dict, Iterator, List, Set, Tuple, TypeVar, Type
 
-import univention.uldap
-import univention.admin.uldap
 import univention.admin.modules as udm_modules
-
-import univention.config_registry
-from univention.management.console.modules.diagnostic import Warning, MODULE
-from univention.management.console.modules.diagnostic import util
-
+import univention.admin.uldap
+import univention.uldap
+from univention.admin.handlers import simpleLdap
+from univention.config_registry import ucr_live as configRegistry
 from univention.lib.i18n import Translation
+from univention.lib.misc import custom_username
+from univention.management.console.modules.diagnostic import MODULE, Instance, Warning, util
+
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check IMAP shared folder ACLs')
 description = _('All shared folder ACLs are in sync with UDM.')
 run_descr = ['Checks if all IMAP shared Folder ACLs are in sync with UDM']
 
+_T = TypeVar("_T", bound="ACLDifferenceError")
+
 
 class ACLError(Exception):
 	pass
 
 
 class MailboxNotExistentError(ACLError):
-	def __init__(self, mailbox):
+	def __init__(self, mailbox: str) -> None:
 		super(ACLError, self).__init__(mailbox)
 		self.mailbox = mailbox
 
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('Mail folder {folder!r} does not exist in IMAP.')
 		return msg.format(folder=self.mailbox)
 
 
 class ACLIdentifierError(Exception):
-	def __init__(self, identifier):
+	def __init__(self, identifier: str) -> None:
 		super(ACLIdentifierError, self).__init__(identifier)
 		self.identifier = identifier
 
 
 class DuplicateIdentifierACLError(ACLIdentifierError):
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('Multiple ACL entries for {id!r} in UDM.')
 		return msg.format(id=self.identifier)
 
 
 class ACLDifferenceError(ACLIdentifierError):
-	def __init__(self, identifier, udm_right, actual_right):
+	def __init__(self, identifier: str, udm_right, actual_right) -> None:
 		super(ACLDifferenceError, self).__init__(identifier)
 		self.udm_right = udm_right
 		self.actual_right = actual_right
 
 
 class UserACLError(ACLDifferenceError):
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('ACL right for user {id!r} is {udm!r} in UDM, but {imap!r} in IMAP.')
 		return msg.format(id=self.identifier, udm=self.udm_right, imap=self.actual_right)
 
 
 class GroupACLError(ACLDifferenceError):
-	def __str__(self):
+	def __str__(self) -> str:
 		msg = _('ACL right for group {id!r} is {udm!r} in UDM, but {imap!r} in IMAP.')
 		return msg.format(id=self.identifier, udm=self.udm_right, imap=self.actual_right)
 
 
 class MailFolder(object):
-	def __init__(self, udm_folder):
+	def __init__(self, udm_folder: simpleLdap) -> None:
 		self.dn = udm_folder.dn
 		self.name = udm_folder.get('name')
 		self.mail_domain = udm_folder.get('mailDomain')
@@ -108,14 +107,14 @@ class MailFolder(object):
 		self._group_acl = udm_folder.get('sharedFolderGroupACL')
 
 	@property
-	def common_name(self):
+	def common_name(self) -> str:
 		return '{}@{}'.format(self.name, self.mail_domain)
 
-	def acl(self):
+	def acl(self) -> "ACL":
 		return ACL.from_udm(self._user_acl, self._group_acl)
 
 	@classmethod
-	def from_udm(cls):
+	def from_udm(cls) -> Iterator["MailFolder"]:
 		univention.admin.modules.update()
 		(ldap_connection, position) = univention.admin.uldap.getMachineConnection()
 		module = udm_modules.get('mail/folder')
@@ -127,34 +126,36 @@ class MailFolder(object):
 class ACL(object):
 	RIGHTS = ('all', 'write', 'append', 'post', 'read', 'none')
 
-	def __init__(self, user_acl, group_acl):
+	def __init__(self, user_acl: Dict[str, str], group_acl: Dict[str, str]):
 		self.user_acl = user_acl
 		self.group_acl = group_acl
 
 	@classmethod
-	def from_udm(cls, user_acl, group_acl):
+	def from_udm(cls, user_acl: List[Tuple[str, str]], group_acl: List[Tuple[str, str]]) -> "ACL":
 		'''
 		Transform the udm acls from [[id, right], [id, right], ..] to a dict
 		from identifier to right, where right is the highest right in the acl.
 		'''
-		def simplify(acl_list):
-			merged = dict()
+		def simplify(acl_list: List[Tuple[str, str]]) -> Iterator[Tuple[str, str]]:
+			merged: Dict[str, Set[str]] = {}
 			for (identifier, right) in acl_list:
 				merged.setdefault(identifier, set()).add(right)
+
 			for (identifier, rights) in merged.items():
 				if len(rights) > 1:
 					raise DuplicateIdentifierACLError(identifier)
 				else:
 					udm_right = next(right for right in cls.RIGHTS if right in rights)
 					yield (identifier, udm_right)
+
 		return cls(dict(simplify(user_acl)), dict(simplify(group_acl)))
 
-	def difference(self, other):
+	def difference(self, other: "ACL") -> Iterator[ACLDifferenceError]:
 		user_diff = self._diff(UserACLError, self.user_acl, other.user_acl)
 		group_diff = self._diff(GroupACLError, self.group_acl, other.group_acl)
 		return it.chain(user_diff, group_diff)
 
-	def _diff(self, exception, expected, actual):
+	def _diff(self, exception: Type[_T], expected, actual) -> Iterator[_T]:
 		all_id = expected.keys() | actual.keys()
 		for identifier in all_id:
 			exp = expected.get(identifier, 'none')
@@ -164,21 +165,21 @@ class ACL(object):
 
 
 class DovecotACL(ACL):
-	DOVECOT_RIGHT_TRANSLATION = (
-		('all', set(('lookup', 'read', 'write', 'write-seen', 'post', 'insert',
-			'write-deleted', 'expunge', 'admin'))),
-		('write', set(('lookup', 'read', 'write', 'write-seen', 'post', 'insert',
-			'write-deleted', 'expunge'))),
-		('append', set(('lookup', 'read', 'write', 'write-seen', 'post', 'insert'))),
-		('post', set(('lookup', 'read', 'write', 'write-seen', 'post'))),
-		('read', set(('lookup', 'read', 'write', 'write-seen'))),
+	DOVECOT_RIGHT_TRANSLATION: Tuple[Tuple[str, Set[str]], ...] = (
+		('all', {'lookup', 'read', 'write', 'write-seen', 'post', 'insert',
+			'write-deleted', 'expunge', 'admin'}),
+		('write', {'lookup', 'read', 'write', 'write-seen', 'post', 'insert',
+			'write-deleted', 'expunge'}),
+		('append', {'lookup', 'read', 'write', 'write-seen', 'post', 'insert'}),
+		('post', {'lookup', 'read', 'write', 'write-seen', 'post'}),
+		('read', {'lookup', 'read', 'write', 'write-seen'}),
 		('none', set()),
 	)
 
 	@classmethod
 	def from_folder(cls, folder):
 		acl_list = cls._get_dovecot_acl(folder)
-		merged = dict()
+		merged = {}
 		for (identifier, rights) in acl_list.items():
 			acl_type = 'group' if identifier.startswith('group=') else 'user'
 			udm_id = identifier.replace('user=', '', 1) if identifier.startswith('user=') \
@@ -189,14 +190,15 @@ class DovecotACL(ACL):
 		return cls(merged.get('user', {}), merged.get('group', {}))
 
 	@staticmethod
-	def _get_dovecot_acl(folder):
+	def _get_dovecot_acl(folder: Any) -> Dict[str, Set]:
 		mailbox = 'shared/{pm}' if folder.mail_address else '{cn}/INBOX'
-		cmd = ('doveadm', 'acl', 'get', '-u', 'Administrator', mailbox.format(cn=folder.common_name, pm=folder.mail_address))
+		admin_user = custom_username("Administrator", configRegistry)
+		cmd = ('doveadm', 'acl', 'get', '-u', admin_user, mailbox.format(cn=folder.common_name, pm=folder.mail_address))
 		output = subprocess.check_output(cmd, stderr=subprocess.PIPE).decode('UTF-8', 'replace').splitlines()
 		return {identifier.strip(): set(rights.strip().split()) for (identifier, rights) in (line.rsplit('  ', 1) for line in output)}
 
 
-def all_differences(acl_class):
+def all_differences(acl_class: Type[DovecotACL]) -> Iterator[Tuple]:
 	for folder in MailFolder.from_udm():
 		try:
 			udm_acl = folder.acl()
@@ -207,7 +209,7 @@ def all_differences(acl_class):
 			yield (folder, error)
 
 
-def udm_mail_link(folder):
+def udm_mail_link(folder) -> Dict[str, Any]:
 	return {
 		'module': 'udm',
 		'flavor': 'mail/mail',
@@ -220,25 +222,22 @@ def udm_mail_link(folder):
 	}
 
 
-def run(_umc_instance):
+def run(_umc_instance: Instance) -> None:
 	if not util.is_service_active('IMAP'):
 		return
 
-	configRegistry = univention.config_registry.ConfigRegistry()
-	configRegistry.load()
-
 	if configRegistry.is_true('mail/dovecot'):
 		acl_class = DovecotACL
 	else:
 		return
 
 	differences = list(all_differences(acl_class))
-	ed = ' '.join([
+	ed = [
 		_('Found differences in the ACLs for IMAP shared folders between UDM and IMAP.'),
 		_('This is not necessarily a problem, if the the ACL got changed via IMAP.')
-	])
+	]
 
-	modules = list()
+	modules = []
 	for (folder, group) in it.groupby(differences, lambda x: x[0]):
 		name = folder.common_name
 		ed.append('')
